/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -i dbustypes.h -p datamanagementinterface /home/trueg/kde/dev/kde/src/kde-runtime/nepomuk/interfaces/org.kde.nepomuk.DataManagement.xml
 *
 * qdbusxml2cpp is Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "datamanagementinterface.h"

/*
 * Implementation of interface class OrgKdeNepomukDataManagementInterface
 */

OrgKdeNepomukDataManagementInterface::OrgKdeNepomukDataManagementInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent)
    : AbstractTimeoutDBusInterface(service, path, staticInterfaceName(), connection, parent)
{
}

OrgKdeNepomukDataManagementInterface::~OrgKdeNepomukDataManagementInterface()
{
}


#include "datamanagementinterface.moc"
