/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -p resourcewatcherconnectioninterface /home/chrigi/devel/kde/kdepim-runtime/agents/nepomukfeeder/dms-copy/interfaces/org.kde.nepomuk.ResourceWatcherConnection.xml
 *
 * qdbusxml2cpp is Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef RESOURCEWATCHERCONNECTIONINTERFACE_H_1312387679
#define RESOURCEWATCHERCONNECTIONINTERFACE_H_1312387679

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.nepomuk.ResourceWatcherConnection
 */
class OrgKdeNepomukResourceWatcherConnectionInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.nepomuk.ResourceWatcherConnection"; }

public:
    OrgKdeNepomukResourceWatcherConnectionInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = 0);

    ~OrgKdeNepomukResourceWatcherConnectionInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> close()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QLatin1String("close"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void propertyAdded(const QString &resource, const QString &property, const QDBusVariant &value);
    void propertyRemoved(const QString &resource, const QString &property, const QDBusVariant &value);
    void resourceCreated(const QString &uri, const QStringList &types);
    void resourceRemoved(const QString &uri, const QStringList &types);
    void resourceTypeAdded(const QString &resUri, const QString &type);
    void resourceTypeRemoved(const QString &resUri, const QString &type);
};

namespace org {
  namespace kde {
    namespace nepomuk {
      typedef ::OrgKdeNepomukResourceWatcherConnectionInterface ResourceWatcherConnection;
    }
  }
}
#endif
