/*
    Copyright (c) 2009 Andras Mantia <amantia@kde.org>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "calendarhandler.h"
#include "event.h"

#include <KCalCore/Event>
#include <KCalCore/CalFormat>

#include <kdebug.h>
#include <kmime/kmime_codecs.h>

#include <KLocale>

#include <QBuffer>
#include <QDomDocument>


CalendarHandler::CalendarHandler( const Akonadi::Collection &imapCollection ) : IncidenceHandler( imapCollection )
{
  m_mimeType = "application/x-vnd.kolab.event";
}


CalendarHandler::~CalendarHandler()
{
}

KCalCore::Incidence::Ptr CalendarHandler::incidenceFromKolab(const KMime::Message::Ptr &data)
{
  return incidenceFromKolabImpl<KCalCore::Event::Ptr, Kolab::Event>( data );
}

QByteArray CalendarHandler::incidenceToXml(const KCalCore::Incidence::Ptr &incidence)
{
  return Kolab::Event::eventToXML( incidence.dynamicCast<KCalCore::Event>(), m_calendar.timeZoneId() ).toUtf8();
}

QStringList CalendarHandler::contentMimeTypes()
{
  return QStringList() << KCalCore::Event::eventMimeType();
}

QString CalendarHandler::iconName() const
{
  return QString::fromLatin1( "view-calendar" );
}
